@@ -140,13 +140,6 @@ class USBError(Exception):
         return '%s [%s]' % (libusb_error.get(self.value, 'Unknown error'),
                             self.value)
 
-if sys.version_info[0] == 3:
-    _string_item_to_int = lambda x: x
-    _empty_char_p = bytes()
-else:
-    _string_item_to_int = ord
-    _empty_char_p = ''
-
 c_uchar = c_uint8
 c_int_p = POINTER(c_int)
 
@@ -737,7 +730,7 @@ libusb_set_debug.restype = None
 try:
     libusb_get_version = libusb.libusb_get_version
 except AttributeError:
-    _dummy_version = libusb_version(0, 0, 0, 0, _empty_char_p, _empty_char_p)
+    _dummy_version = libusb_version(0, 0, 0, 0, b'', b'')
     _dummy_version_p = pointer(_dummy_version)
     def libusb_get_version():
         return _dummy_version_p
@@ -963,7 +956,7 @@ else:
 
 def libusb_control_transfer_get_data(transfer_p):
     transfer = transfer_p.contents
-    return buffer_at(transfer.buffer.value, transfer.length)[
+    return buffer_at(transfer.buffer, transfer.length)[
         LIBUSB_CONTROL_SETUP_SIZE:]
 
 def libusb_control_transfer_get_setup(transfer_p):
@@ -1103,10 +1096,10 @@ def get_extra(descriptor):
     result = []
     extra_length = descriptor.extra_length
     if extra_length:
-        extra = buffer_at(descriptor.extra.value, extra_length)
+        extra = buffer_at(descriptor.extra, extra_length)
         append = result.append
         while extra:
-            length = _string_item_to_int(extra[0])
+            length = extra[0]
             if not 0 < length <= len(extra):
                 raise ValueError(
                     'Extra descriptor %i is incomplete/invalid' % (
